@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
 */
 
 #ifndef __LIBV4L1_H
@@ -26,6 +26,7 @@ extern "C" {
 #include <stdio.h>
 #include <stdint.h>
 #include <unistd.h>
+#include <libv4l1-videodev.h>
 
 #if __GNUC__ >= 4
 #define LIBV4L_PUBLIC __attribute__ ((visibility("default")))
