@@ -40,14 +40,16 @@
 
 #define CTRL_FLAG_DISABLED (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_INACTIVE | V4L2_CTRL_FLAG_GRABBED)
 
-static bool is_valid_type(enum v4l2_ctrl_type type)
+static bool is_valid_type(__u32 type)
 {
 	switch (type) {
 	case V4L2_CTRL_TYPE_INTEGER:
 	case V4L2_CTRL_TYPE_BOOLEAN:
 	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
 	case V4L2_CTRL_TYPE_BUTTON:
 	case V4L2_CTRL_TYPE_INTEGER64:
+	case V4L2_CTRL_TYPE_BITMASK:
 	case V4L2_CTRL_TYPE_CTRL_CLASS:
 	case V4L2_CTRL_TYPE_STRING:
 		return true;
@@ -232,6 +234,7 @@ void ApplicationWindow::addCtrl(QGridLayout *grid, const v4l2_queryctrl &qctrl)
 			m_widgetMap[qctrl.id] = spin = new QSpinBox(p);
 			spin->setMinimum(qctrl.minimum);
 			spin->setMaximum(qctrl.maximum);
+			spin->setSingleStep(qctrl.step);
 			addWidget(grid, m_widgetMap[qctrl.id]);
 			connect(m_widgetMap[qctrl.id], SIGNAL(valueChanged(int)),
 				m_sigMapper, SLOT(map()));
@@ -260,6 +263,18 @@ void ApplicationWindow::addCtrl(QGridLayout *grid, const v4l2_queryctrl &qctrl)
 				m_sigMapper, SLOT(map()));
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		addLabel(grid, name);
+		edit = new QLineEdit(p);
+		edit->setInputMask("HHHHHHHH");
+		addWidget(grid, edit);
+		m_widgetMap[qctrl.id] = edit;
+		connect(m_widgetMap[qctrl.id], SIGNAL(lostFocus()),
+				m_sigMapper, SLOT(map()));
+		connect(m_widgetMap[qctrl.id], SIGNAL(returnPressed()),
+				m_sigMapper, SLOT(map()));
+		break;
+
 	case V4L2_CTRL_TYPE_STRING:
 		addLabel(grid, name);
 		edit = new QLineEdit(p);
@@ -289,6 +304,7 @@ void ApplicationWindow::addCtrl(QGridLayout *grid, const v4l2_queryctrl &qctrl)
 		break;
 
 	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
 		addLabel(grid, name);
 		combo = new QComboBox(p);
 		m_widgetMap[qctrl.id] = combo;
@@ -297,7 +313,10 @@ void ApplicationWindow::addCtrl(QGridLayout *grid, const v4l2_queryctrl &qctrl)
 			qmenu.index = i;
 			if (!querymenu(qmenu))
 				continue;
-			combo->addItem((char *)qmenu.name);
+			if (qctrl.type == V4L2_CTRL_TYPE_MENU)
+				combo->addItem((char *)qmenu.name);
+			else
+				combo->addItem(QString("%1").arg(qmenu.value));
 		}
 		addWidget(grid, m_widgetMap[qctrl.id]);
 		connect(m_widgetMap[qctrl.id], SIGNAL(activated(int)),
@@ -455,7 +474,11 @@ int ApplicationWindow::getVal(unsigned id)
 		v = static_cast<QCheckBox *>(w)->isChecked();
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		v = (int)static_cast<QLineEdit *>(w)->text().toUInt(0, 16);
+		break;
 	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
 		idx = static_cast<QComboBox *>(w)->currentIndex();
 		for (i = qctrl.minimum; i <= qctrl.maximum; i++) {
 			qmenu.id = qctrl.id;
@@ -647,6 +670,16 @@ void ApplicationWindow::setWhat(QWidget *w, unsigned id, long long v)
 		w->setStatusTip(w->whatsThis());
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		w->setWhatsThis(QString("Type: Bitmask\n"
+					"Maximum: %1\n"
+					"Current: %2\n"
+					"Default: %3")
+			.arg((unsigned)qctrl.maximum, 0, 16).arg((unsigned)v, 0, 16)
+			.arg((unsigned)qctrl.default_value, 0, 16) + flags);
+		w->setStatusTip(w->whatsThis());
+		break;
+
 	case V4L2_CTRL_TYPE_BUTTON:
 		w->setWhatsThis(QString("Type: Button") + flags);
 		w->setStatusTip(w->whatsThis());
@@ -669,6 +702,16 @@ void ApplicationWindow::setWhat(QWidget *w, unsigned id, long long v)
 			.arg(qctrl.minimum).arg(qctrl.maximum).arg(v).arg(qctrl.default_value) + flags);
 		w->setStatusTip(w->whatsThis());
 		break;
+
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
+		w->setWhatsThis(QString("Type: Integer Menu\n"
+					"Minimum: %1\n"
+					"Maximum: %2\n"
+					"Current: %3\n"
+					"Default: %4")
+			.arg(qctrl.minimum).arg(qctrl.maximum).arg(v).arg(qctrl.default_value) + flags);
+		w->setStatusTip(w->whatsThis());
+		break;
 	default:
 		break;
 	}
@@ -691,11 +734,16 @@ void ApplicationWindow::setVal(unsigned id, int v)
 			static_cast<QLineEdit *>(w)->setText(QString::number(v));
 		break;
 
+	case V4L2_CTRL_TYPE_BITMASK:
+		static_cast<QLineEdit *>(w)->setText(QString("%1").arg((unsigned)v, 8, 16, QChar('0')));
+		break;
+
 	case V4L2_CTRL_TYPE_BOOLEAN:
 		static_cast<QCheckBox *>(w)->setChecked(v);
 		break;
 
 	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
 		idx = 0;
 		for (i = qctrl.minimum; i <= v; i++) {
 			qmenu.id = id;
@@ -754,7 +802,7 @@ void ApplicationWindow::setDefaults(unsigned ctrl_class)
 		if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_INTEGER64)
 			setVal64(id, 0);
 		else if (m_ctrlMap[id].type == V4L2_CTRL_TYPE_STRING)
-			setString(id, QString(' ', m_ctrlMap[id].minimum));
+			setString(id, QString(m_ctrlMap[id].minimum, ' '));
 		else if (m_ctrlMap[id].type != V4L2_CTRL_TYPE_BUTTON)
 			setVal(id, m_ctrlMap[id].default_value);
 	}
@@ -773,6 +821,8 @@ QString ApplicationWindow::getCtrlFlags(unsigned flags)
 		s += "update ";
 	if (flags & V4L2_CTRL_FLAG_INACTIVE)
 		s += "inactive ";
+	if (flags & V4L2_CTRL_FLAG_VOLATILE)
+		s += "volatile ";
 	if (flags & V4L2_CTRL_FLAG_SLIDER)
 		s += "slider ";
 	if (s.length()) s = QString("\nFlags: ") + s;
