@@ -19,6 +19,7 @@
 
 #include "qv4l2.h"
 #include "general-tab.h"
+#include "vbi-tab.h"
 #include "capture-win.h"
 
 #include <QToolBar>
@@ -59,23 +60,40 @@ ApplicationWindow::ApplicationWindow() :
 	m_frameData = NULL;
 	m_nbuffers = 0;
 	m_buffers = NULL;
+	m_makeSnapshot = false;
 
-	QAction *openAct = new QAction(QIcon(":/fileopen.png"), "&Open device", this);
+	QAction *openAct = new QAction(QIcon(":/fileopen.png"), "&Open Device", this);
 	openAct->setStatusTip("Open a v4l device, use libv4l2 wrapper if possible");
 	openAct->setShortcut(Qt::CTRL+Qt::Key_O);
 	connect(openAct, SIGNAL(triggered()), this, SLOT(opendev()));
 
-	QAction *openRawAct = new QAction(QIcon(":/fileopen.png"), "Open &raw device", this);
+	QAction *openRawAct = new QAction(QIcon(":/fileopen.png"), "Open &Raw Device", this);
 	openRawAct->setStatusTip("Open a v4l device without using the libv4l2 wrapper");
 	openRawAct->setShortcut(Qt::CTRL+Qt::Key_R);
 	connect(openRawAct, SIGNAL(triggered()), this, SLOT(openrawdev()));
 
-	m_capStartAct = new QAction(QIcon(":/record.png"), "&Start capturing", this);
+	m_capStartAct = new QAction(QIcon(":/record.png"), "&Start Capturing", this);
 	m_capStartAct->setStatusTip("Start capturing");
 	m_capStartAct->setCheckable(true);
 	m_capStartAct->setDisabled(true);
 	connect(m_capStartAct, SIGNAL(toggled(bool)), this, SLOT(capStart(bool)));
 
+	m_snapshotAct = new QAction(QIcon(":/snapshot.png"), "&Make Snapshot", this);
+	m_snapshotAct->setStatusTip("Make snapshot");
+	m_snapshotAct->setDisabled(true);
+	connect(m_snapshotAct, SIGNAL(triggered()), this, SLOT(snapshot()));
+
+	m_saveRawAct = new QAction(QIcon(":/saveraw.png"), "Save Raw Frames", this);
+	m_saveRawAct->setStatusTip("Save raw frames to file.");
+	m_saveRawAct->setCheckable(true);
+	m_saveRawAct->setChecked(false);
+	connect(m_saveRawAct, SIGNAL(toggled(bool)), this, SLOT(saveRaw(bool)));
+
+	m_showFramesAct = new QAction(QIcon(":/video-television.png"), "Show &Frames", this);
+	m_showFramesAct->setStatusTip("Only show captured frames if set.");
+	m_showFramesAct->setCheckable(true);
+	m_showFramesAct->setChecked(true);
+
 	QAction *closeAct = new QAction(QIcon(":/fileclose.png"), "&Close", this);
 	closeAct->setStatusTip("Close");
 	closeAct->setShortcut(Qt::CTRL+Qt::Key_W);
@@ -91,6 +109,9 @@ ApplicationWindow::ApplicationWindow() :
 	fileMenu->addAction(openRawAct);
 	fileMenu->addAction(closeAct);
 	fileMenu->addAction(m_capStartAct);
+	fileMenu->addAction(m_snapshotAct);
+	fileMenu->addAction(m_saveRawAct);
+	fileMenu->addAction(m_showFramesAct);
 	fileMenu->addSeparator();
 	fileMenu->addAction(quitAct);
 
@@ -98,6 +119,9 @@ ApplicationWindow::ApplicationWindow() :
 	toolBar->setObjectName("toolBar");
 	toolBar->addAction(openAct);
 	toolBar->addAction(m_capStartAct);
+	toolBar->addAction(m_snapshotAct);
+	toolBar->addAction(m_saveRawAct);
+	toolBar->addAction(m_showFramesAct);
 	toolBar->addSeparator();
 	toolBar->addAction(quitAct);
 
@@ -139,13 +163,19 @@ void ApplicationWindow::setDevice(const QString &device, bool rawOpen)
 	m_genTab = new GeneralTab(device, *this, 4, w);
 	m_tabs->addTab(w, "General");
 	addTabs();
+	if (caps() & (V4L2_CAP_VBI_CAPTURE | V4L2_CAP_SLICED_VBI_CAPTURE)) {
+		w = new QWidget(m_tabs);
+		m_vbiTab = new VbiTab(w);
+		m_tabs->addTab(w, "VBI");
+	}
 	if (QWidget *current = m_tabs->currentWidget()) {
 		current->show();
 	}
+	statusBar()->clearMessage();
 	m_tabs->show();
 	m_tabs->setFocus();
 	m_convertData = v4lconvert_create(fd());
-	m_capStartAct->setEnabled(fd() >= 0);
+	m_capStartAct->setEnabled(fd() >= 0 && !m_genTab->isRadio());
 }
 
 void ApplicationWindow::opendev()
@@ -168,10 +198,113 @@ void ApplicationWindow::openrawdev()
 		setDevice(d.selectedFiles().first(), true);
 }
 
+void ApplicationWindow::capVbiFrame()
+{
+	__u32 buftype = m_genTab->bufType();
+	v4l2_buffer buf;
+	__u8 *data = NULL;
+	int s = 0;
+	bool again;
+
+	switch (m_capMethod) {
+	case methodRead:
+		s = read(m_frameData, m_vbiSize);
+		if (s < 0) {
+			if (errno != EAGAIN) {
+				error("read");
+				m_capStartAct->setChecked(false);
+			}
+			return;
+		}
+		data = m_frameData;
+		break;
+
+	case methodMmap:
+		if (!dqbuf_mmap(buf, buftype, again)) {
+			error("dqbuf");
+			m_capStartAct->setChecked(false);
+			return;
+		}
+		if (again)
+			return;
+		data = (__u8 *)m_buffers[buf.index].start;
+		s = buf.bytesused;
+		break;
+
+	case methodUser:
+		if (!dqbuf_user(buf, buftype, again)) {
+			error("dqbuf");
+			m_capStartAct->setChecked(false);
+			return;
+		}
+		if (again)
+			return;
+		data = (__u8 *)buf.m.userptr;
+		s = buf.bytesused;
+		break;
+	}
+	if (buftype == V4L2_BUF_TYPE_VBI_CAPTURE && s != m_vbiSize) {
+		error("incorrect vbi size");
+		m_capStartAct->setChecked(false);
+		return;
+	}
+	if (m_showFrames) {
+		for (unsigned y = 0; y < m_vbiHeight; y++) {
+			__u8 *p = data + y * m_vbiWidth;
+			__u8 *q = m_capImage->bits() + y * m_capImage->bytesPerLine();
+
+			for (unsigned x = 0; x < m_vbiWidth; x++) {
+				*q++ = *p;
+				*q++ = *p;
+				*q++ = *p++;
+			}
+		}
+	}
+
+	struct v4l2_sliced_vbi_format sfmt;
+	struct v4l2_sliced_vbi_data sdata[m_vbiHandle.count[0] + m_vbiHandle.count[1]];
+	struct v4l2_sliced_vbi_data *p;
+
+	if (buftype == V4L2_BUF_TYPE_SLICED_VBI_CAPTURE) {
+		p = (struct v4l2_sliced_vbi_data *)data;
+	} else {
+		vbi_parse(&m_vbiHandle, data, &sfmt, sdata);
+		s = sizeof(sdata);
+		p = sdata;
+	}
+
+	if (m_capMethod != methodRead)
+		qbuf(buf);
+
+	m_vbiTab->slicedData(p, s / sizeof(p[0]));
+
+	QString status, curStatus;
+	struct timeval tv, res;
+
+	if (m_frame == 0)
+		gettimeofday(&m_tv, NULL);
+	gettimeofday(&tv, NULL);
+	timersub(&tv, &m_tv, &res);
+	if (res.tv_sec) {
+		m_fps = (100 * (m_frame - m_lastFrame)) /
+			(res.tv_sec * 100 + res.tv_usec / 10000);
+		m_lastFrame = m_frame;
+		m_tv = tv;
+	}
+	status = QString("Frame: %1 Fps: %2").arg(++m_frame).arg(m_fps);
+	if (m_showFrames)
+		m_capture->setImage(*m_capImage, status);
+	curStatus = statusBar()->currentMessage();
+	if (curStatus.isEmpty() || curStatus.startsWith("Frame: "))
+		statusBar()->showMessage(status);
+	if (m_frame == 1)
+		refresh();
+}
+
 void ApplicationWindow::capFrame()
 {
+	__u32 buftype = m_genTab->bufType();
 	v4l2_buffer buf;
-	unsigned i;
 	int s = 0;
 	int err = 0;
 	bool again;
@@ -186,16 +319,23 @@ void ApplicationWindow::capFrame()
 			}
 			return;
 		}
-		if (useWrapper())
-			memcpy(m_capImage->bits(), m_frameData, s);
-		else
+		if (m_makeSnapshot)
+			makeSnapshot((unsigned char *)m_frameData, s);
+		if (m_saveRaw.openMode())
+			m_saveRaw.write((const char *)m_frameData, s);
+
+		if (!m_showFrames)
+			break;
+		if (m_mustConvert)
 			err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
 				m_frameData, s,
 				m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+		if (!m_mustConvert || err < 0)
+			memcpy(m_capImage->bits(), m_frameData, std::min(s, m_capImage->numBytes()));
 		break;
 
 	case methodMmap:
-		if (!dqbuf_mmap_cap(buf, again)) {
+		if (!dqbuf_mmap(buf, buftype, again)) {
 			error("dqbuf");
 			m_capStartAct->setChecked(false);
 			return;
@@ -203,19 +343,27 @@ void ApplicationWindow::capFrame()
 		if (again)
 			return;
 
-		if (useWrapper())
-			memcpy(m_capImage->bits(), (unsigned char *)m_buffers[buf.index].start,
-					buf.bytesused);
-		else
-			err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
-				(unsigned char *)m_buffers[buf.index].start, buf.bytesused,
-				m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+		if (m_showFrames) {
+			if (m_mustConvert)
+				err = v4lconvert_convert(m_convertData,
+					&m_capSrcFormat, &m_capDestFormat,
+					(unsigned char *)m_buffers[buf.index].start, buf.bytesused,
+					m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+			if (!m_mustConvert || err < 0)
+				memcpy(m_capImage->bits(),
+				       (unsigned char *)m_buffers[buf.index].start,
+				       std::min(buf.bytesused, (unsigned)m_capImage->numBytes()));
+		}
+		if (m_makeSnapshot)
+			makeSnapshot((unsigned char *)m_buffers[buf.index].start, buf.bytesused);
+		if (m_saveRaw.openMode())
+			m_saveRaw.write((const char *)m_buffers[buf.index].start, buf.bytesused);
 
 		qbuf(buf);
 		break;
 
 	case methodUser:
-		if (!dqbuf_user_cap(buf, again)) {
+		if (!dqbuf_user(buf, buftype, again)) {
 			error("dqbuf");
 			m_capStartAct->setChecked(false);
 			return;
@@ -223,51 +371,73 @@ void ApplicationWindow::capFrame()
 		if (again)
 			return;
 
-		for (i = 0; i < m_nbuffers; ++i)
-			if (buf.m.userptr == (unsigned long)m_buffers[i].start
-					&& buf.length == m_buffers[i].length)
-				break;
-
-		if (useWrapper())
-			memcpy(m_capImage->bits(), (unsigned char *)buf.m.userptr,
-					buf.bytesused);
-		else
-			err = v4lconvert_convert(m_convertData, &m_capSrcFormat, &m_capDestFormat,
-				(unsigned char *)buf.m.userptr, buf.bytesused,
-				m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+		if (m_showFrames) {
+			if (m_mustConvert)
+				err = v4lconvert_convert(m_convertData,
+					&m_capSrcFormat, &m_capDestFormat,
+					(unsigned char *)buf.m.userptr, buf.bytesused,
+					m_capImage->bits(), m_capDestFormat.fmt.pix.sizeimage);
+			if (!m_mustConvert || err < 0)
+				memcpy(m_capImage->bits(), (unsigned char *)buf.m.userptr,
+				       std::min(buf.bytesused, (unsigned)m_capImage->numBytes()));
+		}
+		if (m_makeSnapshot)
+			makeSnapshot((unsigned char *)buf.m.userptr, buf.bytesused);
+		if (m_saveRaw.openMode())
+			m_saveRaw.write((const char *)buf.m.userptr, buf.bytesused);
 
 		qbuf(buf);
 		break;
 	}
-	if (err == -1)
+	if (err == -1 && m_frame == 0)
 		error(v4lconvert_get_error_message(m_convertData));
 
-	m_capture->setImage(*m_capImage);
-	if (m_capture->frame() == 1)
+	QString status, curStatus;
+	struct timeval tv, res;
+
+	if (m_frame == 0)
+		gettimeofday(&m_tv, NULL);
+	gettimeofday(&tv, NULL);
+	timersub(&tv, &m_tv, &res);
+	if (res.tv_sec) {
+		m_fps = (100 * (m_frame - m_lastFrame)) /
+			(res.tv_sec * 100 + res.tv_usec / 10000);
+		m_lastFrame = m_frame;
+		m_tv = tv;
+	}
+	status = QString("Frame: %1 Fps: %2").arg(++m_frame).arg(m_fps);
+	if (m_showFrames)
+		m_capture->setImage(*m_capImage, status);
+	curStatus = statusBar()->currentMessage();
+	if (curStatus.isEmpty() || curStatus.startsWith("Frame: "))
+		statusBar()->showMessage(status);
+	if (m_frame == 1)
 		refresh();
 }
 
 bool ApplicationWindow::startCapture(unsigned buffer_size)
 {
-	unsigned int i;
+	__u32 buftype = m_genTab->bufType();
 	v4l2_requestbuffers req;
+	unsigned int i;
 
 	memset(&req, 0, sizeof(req));
 
 	switch (m_capMethod) {
 	case methodRead:
+		m_snapshotAct->setEnabled(true);
 		/* Nothing to do. */
 		return true;
 
 	case methodMmap:
-		if (!reqbufs_mmap_cap(req, 3)) {
+		if (!reqbufs_mmap(req, buftype, 3)) {
 			error("Cannot capture");
 			break;
 		}
 
 		if (req.count < 2) {
 			error("Too few buffers");
-			reqbufs_mmap_cap(req);
+			reqbufs_mmap(req, buftype);
 			break;
 		}
 
@@ -275,7 +445,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 
 		if (!m_buffers) {
 			error("Out of memory");
-			reqbufs_mmap_cap(req);
+			reqbufs_mmap(req, buftype);
 			break;
 		}
 
@@ -284,7 +454,7 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 
 			memset(&buf, 0, sizeof(buf));
 
-			buf.type        = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+			buf.type        = buftype;
 			buf.memory      = V4L2_MEMORY_MMAP;
 			buf.index       = m_nbuffers;
 
@@ -302,26 +472,27 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 			}
 		}
 		for (i = 0; i < m_nbuffers; ++i) {
-			if (!qbuf_mmap_cap(i)) {
+			if (!qbuf_mmap(i, buftype)) {
 				perror("VIDIOC_QBUF");
 				goto error;
 			}
 		}
-		if (!streamon_cap()) {
+		if (!streamon(buftype)) {
 			perror("VIDIOC_STREAMON");
 			goto error;
 		}
+		m_snapshotAct->setEnabled(true);
 		return true;
 
 	case methodUser:
-		if (!reqbufs_user_cap(req, 3)) {
+		if (!reqbufs_user(req, buftype, 3)) {
 			error("Cannot capture");
 			break;
 		}
 
 		if (req.count < 2) {
 			error("Too few buffers");
-			reqbufs_user_cap(req);
+			reqbufs_user(req, buftype);
 			break;
 		}
 
@@ -342,14 +513,15 @@ bool ApplicationWindow::startCapture(unsigned buffer_size)
 			}
 		}
 		for (i = 0; i < m_nbuffers; ++i)
-			if (!qbuf_user_cap(i, m_buffers[i].start, m_buffers[i].length)) {
+			if (!qbuf_user(i, buftype, m_buffers[i].start, m_buffers[i].length)) {
 				perror("VIDIOC_QBUF");
 				goto error;
 			}
-		if (!streamon_cap()) {
+		if (!streamon(buftype)) {
 			perror("VIDIOC_STREAMON");
 			goto error;
 		}
+		m_snapshotAct->setEnabled(true);
 		return true;
 	}
 
@@ -360,10 +532,12 @@ error:
 
 void ApplicationWindow::stopCapture()
 {
+	__u32 buftype = m_genTab->bufType();
 	v4l2_requestbuffers reqbufs;
 	v4l2_encoder_cmd cmd;
 	unsigned i;
 
+	m_snapshotAct->setDisabled(true);
 	switch (m_capMethod) {
 	case methodRead:
 		memset(&cmd, 0, sizeof(cmd));
@@ -374,22 +548,22 @@ void ApplicationWindow::stopCapture()
 	case methodMmap:
 		if (m_buffers == NULL)
 			break;
-		if (!streamoff_cap())
+		if (!streamoff(buftype))
 			perror("VIDIOC_STREAMOFF");
 		for (i = 0; i < m_nbuffers; ++i)
 			if (-1 == munmap(m_buffers[i].start, m_buffers[i].length))
 				perror("munmap");
 		// Free all buffers.
-		reqbufs_mmap_cap(reqbufs, 1);  // videobuf workaround
-		reqbufs_mmap_cap(reqbufs, 0);
+		reqbufs_mmap(reqbufs, buftype, 1);  // videobuf workaround
+		reqbufs_mmap(reqbufs, buftype, 0);
 		break;
 
 	case methodUser:
-		if (!streamoff_cap())
+		if (!streamoff(buftype))
 			perror("VIDIOC_STREAMOFF");
 		// Free all buffers.
-		reqbufs_user_cap(reqbufs, 1);  // videobuf workaround
-		reqbufs_user_cap(reqbufs, 0);
+		reqbufs_user(reqbufs, buftype, 1);  // videobuf workaround
+		reqbufs_user(reqbufs, buftype, 0);
 		for (i = 0; i < m_nbuffers; ++i)
 			free(m_buffers[i].start);
 		break;
@@ -418,58 +592,133 @@ void ApplicationWindow::capStart(bool start)
 		__u32 v4l2_pixfmt;
 		QImage::Format qt_pixfmt;
 	} supported_fmts[] = {
-		{ V4L2_PIX_FMT_RGB32, QImage::Format_ARGB32 },
+#if Q_BYTE_ORDER == Q_BIG_ENDIAN
+		{ V4L2_PIX_FMT_RGB32, QImage::Format_RGB32 },
 		{ V4L2_PIX_FMT_RGB24, QImage::Format_RGB888 },
 		{ V4L2_PIX_FMT_RGB565X, QImage::Format_RGB16 },
 		{ V4L2_PIX_FMT_RGB555X, QImage::Format_RGB555 },
+#else
+		{ V4L2_PIX_FMT_BGR32, QImage::Format_RGB32 },
+		{ V4L2_PIX_FMT_RGB24, QImage::Format_RGB888 },
+		{ V4L2_PIX_FMT_RGB565, QImage::Format_RGB16 },
+		{ V4L2_PIX_FMT_RGB555, QImage::Format_RGB555 },
 		{ V4L2_PIX_FMT_RGB444, QImage::Format_RGB444 },
+#endif
 		{ 0, QImage::Format_Invalid }
 	};
 	QImage::Format dstFmt = QImage::Format_RGB888;
+	struct v4l2_fract interval;
+	v4l2_pix_format &srcPix = m_capSrcFormat.fmt.pix;
+	v4l2_pix_format &dstPix = m_capDestFormat.fmt.pix;
 
 	if (!start) {
 		stopCapture();
 		delete m_capNotifier;
 		m_capNotifier = NULL;
 		delete m_capImage;
+		m_capImage = NULL;
 		return;
 	}
+	m_showFrames = m_showFramesAct->isChecked();
+	m_frame = m_lastFrame = m_fps = 0;
 	m_capMethod = m_genTab->capMethod();
-	g_fmt_cap(m_capSrcFormat);
-	if (useWrapper() &&
-	    m_capSrcFormat.fmt.pix.pixelformat != V4L2_PIX_FMT_RGB24) {
-	        v4l2_fract interval;
-	        bool interval_ok = get_interval(interval);
-		m_capSrcFormat.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB24;
-		s_fmt(m_capSrcFormat);
-		g_fmt_cap(m_capSrcFormat);
-		if (interval_ok)
-		        set_interval(interval);
-	}
-	m_frameData = new unsigned char[m_capSrcFormat.fmt.pix.sizeimage];
-	m_capDestFormat = m_capSrcFormat;
-	m_capDestFormat.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB24;
 
-	for (int i = 0; supported_fmts[i].v4l2_pixfmt; i++) {
-		if (supported_fmts[i].v4l2_pixfmt == m_capSrcFormat.fmt.pix.pixelformat) {
-			m_capDestFormat.fmt.pix.pixelformat = supported_fmts[i].v4l2_pixfmt;
-			dstFmt = supported_fmts[i].qt_pixfmt;
-			break;
+	if (m_genTab->isSlicedVbi()) {
+		v4l2_format fmt;
+		v4l2_std_id std;
+
+		m_showFrames = false;
+		g_fmt_sliced_vbi(fmt);
+		g_std(std);
+		fmt.fmt.sliced.service_set = (std & V4L2_STD_625_50) ?
+			V4L2_SLICED_VBI_625 : V4L2_SLICED_VBI_525;
+		s_fmt(fmt);
+		memset(&m_vbiHandle, 0, sizeof(m_vbiHandle));
+		m_vbiTab->slicedFormat(fmt.fmt.sliced);
+		m_vbiSize = fmt.fmt.sliced.io_size;
+		m_frameData = new unsigned char[m_vbiSize];
+		if (startCapture(m_vbiSize)) {
+			m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
+			connect(m_capNotifier, SIGNAL(activated(int)), this, SLOT(capVbiFrame()));
 		}
+		return;
 	}
-	if (!useWrapper()) {
-		v4lconvert_try_format(m_convertData, &m_capDestFormat, &m_capSrcFormat);
-		// v4lconvert_try_format sometimes modifies the source format if it thinks
-		// that there is a better format available. Restore our selected source
-		// format since we do not want that happening.
-		g_fmt_cap(m_capSrcFormat);
+	if (m_genTab->isVbi()) {
+		v4l2_format fmt;
+		v4l2_std_id std;
+
+		g_fmt_vbi(fmt);
+		if (fmt.fmt.vbi.sample_format != V4L2_PIX_FMT_GREY) {
+			error("non-grey pixelformat not supported for VBI\n");
+			return;
+		}
+		s_fmt(fmt);
+		g_std(std);
+		if (!vbi_prepare(&m_vbiHandle, &fmt.fmt.vbi, std)) {
+			error("no services possible\n");
+			return;
+		}
+		m_vbiTab->rawFormat(fmt.fmt.vbi);
+		m_vbiWidth = fmt.fmt.vbi.samples_per_line;
+		if (fmt.fmt.vbi.flags & V4L2_VBI_INTERLACED)
+			m_vbiHeight = fmt.fmt.vbi.count[0];
+		else
+			m_vbiHeight = fmt.fmt.vbi.count[0] + fmt.fmt.vbi.count[1];
+		m_vbiSize = m_vbiWidth * m_vbiHeight;
+		m_frameData = new unsigned char[m_vbiSize];
+		if (m_showFrames) {
+			m_capture->setMinimumSize(m_vbiWidth, m_vbiHeight);
+			m_capImage = new QImage(m_vbiWidth, m_vbiHeight, dstFmt);
+			m_capImage->fill(0);
+			m_capture->setImage(*m_capImage, "No frame");
+			m_capture->show();
+		}
+		statusBar()->showMessage("No frame");
+		if (startCapture(m_vbiSize)) {
+			m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
+			connect(m_capNotifier, SIGNAL(activated(int)), this, SLOT(capVbiFrame()));
+		}
+		return;
 	}
-	m_capture->setMinimumSize(m_capDestFormat.fmt.pix.width, m_capDestFormat.fmt.pix.height);
-	m_capImage = new QImage(m_capDestFormat.fmt.pix.width, m_capDestFormat.fmt.pix.height, dstFmt);
-	m_capImage->fill(0);
-	m_capture->setImage(*m_capImage, true);
-	m_capture->show();
-	if (startCapture(m_capSrcFormat.fmt.pix.sizeimage)) {
+
+	g_fmt_cap(m_capSrcFormat);
+	s_fmt(m_capSrcFormat);
+	if (m_genTab->get_interval(interval))
+		set_interval(interval);
+
+	m_mustConvert = m_showFrames;
+	m_frameData = new unsigned char[srcPix.sizeimage];
+	if (m_showFrames) {
+		m_capDestFormat = m_capSrcFormat;
+		dstPix.pixelformat = V4L2_PIX_FMT_RGB24;
+
+		for (int i = 0; supported_fmts[i].v4l2_pixfmt; i++) {
+			if (supported_fmts[i].v4l2_pixfmt == srcPix.pixelformat) {
+				dstPix.pixelformat = supported_fmts[i].v4l2_pixfmt;
+				dstFmt = supported_fmts[i].qt_pixfmt;
+				m_mustConvert = false;
+				break;
+			}
+		}
+		if (m_mustConvert) {
+			v4l2_format copy = m_capSrcFormat;
+
+			v4lconvert_try_format(m_convertData, &m_capDestFormat, &m_capSrcFormat);
+			// v4lconvert_try_format sometimes modifies the source format if it thinks
+			// that there is a better format available. Restore our selected source
+			// format since we do not want that happening.
+			m_capSrcFormat = copy;
+		}
+
+		m_capture->setMinimumSize(dstPix.width, dstPix.height);
+		m_capImage = new QImage(dstPix.width, dstPix.height, dstFmt);
+		m_capImage->fill(0);
+		m_capture->setImage(*m_capImage, "No frame");
+		m_capture->show();
+	}
+
+	statusBar()->showMessage("No frame");
+	if (startCapture(srcPix.sizeimage)) {
 		m_capNotifier = new QSocketNotifier(fd(), QSocketNotifier::Read, m_tabs);
 		connect(m_capNotifier, SIGNAL(activated(int)), this, SLOT(capFrame()));
 	}
@@ -486,6 +735,7 @@ void ApplicationWindow::closeDevice()
 			delete m_capNotifier;
 			delete m_capImage;
 			m_capNotifier = NULL;
+			m_capImage = NULL;
 		}
 		delete m_frameData;
 		m_frameData = NULL;
@@ -503,6 +753,84 @@ void ApplicationWindow::closeDevice()
 	m_classMap.clear();
 }
 
+bool SaveDialog::setBuffer(unsigned char *buf, unsigned size)
+{
+	m_buf = new unsigned char[size];
+	m_size = size;
+	if (m_buf == NULL)
+		return false;
+	memcpy(m_buf, buf, size);
+	return true;
+}
+
+void SaveDialog::selected(const QString &s)
+{
+	if (!s.isEmpty()) {
+		QFile file(s);
+		file.open(QIODevice::WriteOnly | QIODevice::Truncate);
+		file.write((const char *)m_buf, m_size);
+		file.close();
+	}
+	delete [] m_buf;
+}
+
+void ApplicationWindow::makeSnapshot(unsigned char *buf, unsigned size)
+{
+	m_makeSnapshot = false;
+	SaveDialog *dlg = new SaveDialog(this, "Save Snapshot");
+	dlg->setAttribute(Qt::WA_DeleteOnClose);
+	dlg->setFileMode(QFileDialog::AnyFile);
+	dlg->setAcceptMode(QFileDialog::AcceptSave);
+	dlg->setModal(false);
+	if (!dlg->setBuffer(buf, size)) {
+		delete dlg;
+		error("No memory to make snapshot\n");
+		return;
+	}
+	connect(dlg, SIGNAL(fileSelected(const QString &)), dlg, SLOT(selected(const QString &)));
+	dlg->show();
+}
+
+void ApplicationWindow::snapshot()
+{
+	m_makeSnapshot = true;
+}
+
+void ApplicationWindow::rejectedRawFile()
+{
+	m_saveRawAct->setChecked(false);
+}
+
+void ApplicationWindow::openRawFile(const QString &s)
+{
+	if (s.isEmpty())
+		return;
+
+	if (m_saveRaw.openMode())
+		m_saveRaw.close();
+	m_saveRaw.setFileName(s);
+	m_saveRaw.open(QIODevice::WriteOnly | QIODevice::Truncate);
+	m_saveRawAct->setChecked(true);
+}
+
+void ApplicationWindow::saveRaw(bool checked)
+{
+	if (!checked) {
+		if (m_saveRaw.openMode())
+			m_saveRaw.close();
+		return;
+	}
+
+	SaveDialog *dlg = new SaveDialog(this, "Save Raw Frames");
+	dlg->setAttribute(Qt::WA_DeleteOnClose);
+	dlg->setFileMode(QFileDialog::AnyFile);
+	dlg->setAcceptMode(QFileDialog::AcceptSave);
+	dlg->setModal(false);
+	connect(dlg, SIGNAL(fileSelected(const QString &)), this, SLOT(openRawFile(const QString &)));
+	connect(dlg, SIGNAL(rejected()), this, SLOT(rejectedRawFile()));
+	dlg->show();
+}
+
 void ApplicationWindow::about()
 {
 	QMessageBox::about(this, "V4L2 Test Bench",
