@@ -30,7 +30,6 @@
 extern int verbose;
 extern int wrapper;
 extern int kernel_version;
-extern unsigned caps;
 extern unsigned warnings;
 
 struct test_queryctrl: v4l2_queryctrl {
@@ -40,12 +39,19 @@ struct test_queryctrl: v4l2_queryctrl {
 typedef std::list<test_queryctrl> qctrl_list;
 typedef std::set<__u32> pixfmt_set;
 
+struct node;
+
 struct node {
 	int fd;
 	bool is_video;
 	bool is_radio;
 	bool is_vbi;
+	bool is_m2m;
+	bool can_capture;
+	bool can_output;
+	const char *device;
 	unsigned caps;
+	struct node *node2;	/* second open filehandle */
 	bool has_outputs;
 	bool has_inputs;
 	unsigned tuners;
@@ -59,6 +65,8 @@ struct node {
 	qctrl_list controls;
 	__u32 fbuf_caps;
 	pixfmt_set buftype_pixfmts[V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE + 1];
+	__u32 valid_buftypes;
+	__u32 valid_buftype;
 };
 
 #define info(fmt, args...) 					\
@@ -71,7 +79,7 @@ struct node {
 	do {							\
 		warnings++;					\
 		if (verbose)					\
- 			printf("\t\twarn: " fmt, ##args);	\
+ 			printf("\t\twarn: %s(%d): " fmt, __FILE__, __LINE__, ##args);	\
 	} while (0)
 
 #define fail(fmt, args...) 						\
@@ -96,6 +104,16 @@ static inline int test_close(int fd)
 	return wrapper ? v4l2_close(fd) : close(fd);
 }
 
+static inline void reopen(struct node *node)
+{
+	test_close(node->fd);
+	if ((node->fd = test_open(node->device, O_RDWR)) < 0) {
+		fprintf(stderr, "Failed to open %s: %s\n", node->device,
+			strerror(errno));
+		exit(1);
+	}
+}
+
 static inline int test_ioctl(int fd, int cmd, void *arg)
 {
 	return wrapper ? v4l2_ioctl(fd, cmd, arg) : ioctl(fd, cmd, arg);
@@ -136,6 +154,7 @@ int testLogStatus(struct node *node);
 // Input ioctl tests
 int testTuner(struct node *node);
 int testTunerFreq(struct node *node);
+int testTunerHwSeek(struct node *node);
 int testEnumInputAudio(struct node *node);
 int testInput(struct node *node);
 int testInputAudio(struct node *node);
@@ -151,16 +170,31 @@ int testOutputAudio(struct node *node);
 int testQueryControls(struct node *node);
 int testSimpleControls(struct node *node);
 int testExtendedControls(struct node *node);
+int testControlEvents(struct node *node);
+int testJpegComp(struct node *node);
 
 // I/O configuration ioctl tests
 int testStd(struct node *node);
 int testPresets(struct node *node);
-int testCustomTimings(struct node *node);
+int testTimings(struct node *node);
+int testTimingsCap(struct node *node);
 
 // Format ioctl tests
 int testEnumFormats(struct node *node);
+int testParm(struct node *node);
 int testFBuf(struct node *node);
-int testFormats(struct node *node);
+int testGetFormats(struct node *node);
+int testTryFormats(struct node *node);
+int testSetFormats(struct node *node);
 int testSlicedVBICap(struct node *node);
 
+// Codec ioctl tests
+int testEncoder(struct node *node);
+int testEncIndex(struct node *node);
+int testDecoder(struct node *node);
+
+// Buffer ioctl tests
+int testReqBufs(struct node *node);
+int testReadWrite(struct node *node);
+
 #endif
