@@ -34,17 +34,16 @@
 
 static const __u32 buftype2cap[] = {
 	0,
-	V4L2_CAP_VIDEO_CAPTURE,
-	V4L2_CAP_VIDEO_OUTPUT,
+	V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_M2M,
+	V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VIDEO_M2M,
 	V4L2_CAP_VIDEO_OVERLAY,
 	V4L2_CAP_VBI_CAPTURE,
 	V4L2_CAP_VBI_OUTPUT,
 	V4L2_CAP_SLICED_VBI_CAPTURE,
 	V4L2_CAP_SLICED_VBI_OUTPUT,
 	V4L2_CAP_VIDEO_OUTPUT_OVERLAY,
-	// To be discussed
-	V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_VIDEO_CAPTURE,
-	V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_VIDEO_OUTPUT,
+	V4L2_CAP_VIDEO_CAPTURE_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE,
+	V4L2_CAP_VIDEO_OUTPUT_MPLANE | V4L2_CAP_VIDEO_M2M_MPLANE,
 };
 
 static int testEnumFrameIntervals(struct node *node, __u32 pixfmt, __u32 w, __u32 h, bool valid)
@@ -63,10 +62,12 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt, __u32 w, __u3
 		frmival.height = h;
 
 		ret = doioctl(node, VIDIOC_ENUM_FRAMEINTERVALS, &frmival);
+		if (ret == ENOTTY)
+			return ret;
 		if (f == 0 && ret == EINVAL) {
 			if (valid)
 				warn("found framesize %dx%d, but no frame intervals\n", w, h);
-			return -ENOSYS;
+			return ENOTTY;
 		}
 		if (ret == EINVAL)
 			break;
@@ -107,7 +108,7 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt, __u32 w, __u3
 		default:
 			return fail("frmival.type is invalid\n");
 		}
-		
+
 		f++;
 	}
 	if (!valid)
@@ -130,8 +131,10 @@ static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 		frmsize.pixel_format = pixfmt;
 
 		ret = doioctl(node, VIDIOC_ENUM_FRAMESIZES, &frmsize);
+		if (ret == ENOTTY)
+			return ret;
 		if (f == 0 && ret == EINVAL)
-			return -ENOSYS;
+			return ENOTTY;
 		if (ret == EINVAL)
 			break;
 		if (ret)
@@ -149,11 +152,11 @@ static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 				return fail("mixing discrete and stepwise is not allowed\n");
 			ret = testEnumFrameIntervals(node, pixfmt,
 					frmsize.discrete.width, frmsize.discrete.height, true);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			ret = testEnumFrameIntervals(node, pixfmt,
 					frmsize.discrete.width + 1, frmsize.discrete.height, false);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			break;
 		case V4L2_FRMSIZE_TYPE_CONTINUOUS:
@@ -173,32 +176,32 @@ static int testEnumFrameSizes(struct node *node, __u32 pixfmt)
 				return fail("step > max - min for width or height\n");
 			ret = testEnumFrameIntervals(node, pixfmt,
 					sw->min_width, sw->min_height, true);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			ret = testEnumFrameIntervals(node, pixfmt,
 					sw->max_width, sw->max_height, true);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			ret = testEnumFrameIntervals(node, pixfmt,
 					sw->min_width - 1, sw->min_height, false);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			ret = testEnumFrameIntervals(node, pixfmt,
 					sw->max_width, sw->max_height + 1, false);
-			if (ret > 0)
+			if (ret && ret != ENOTTY)
 				return ret;
 			break;
 		default:
 			return fail("frmsize.type is invalid\n");
 		}
-		
+
 		f++;
 	}
 	info("found %d framesizes for pixel format %08x\n", f, pixfmt);
 	return 0;
 }
 
-static int testEnumFormatsType(struct node *node, enum v4l2_buf_type type)
+static int testEnumFormatsType(struct node *node, unsigned type)
 {
 	pixfmt_set &set = node->buftype_pixfmts[type];
 	struct v4l2_fmtdesc fmtdesc;
@@ -211,8 +214,10 @@ static int testEnumFormatsType(struct node *node, enum v4l2_buf_type type)
 		fmtdesc.index = f;
 
 		ret = doioctl(node, VIDIOC_ENUM_FMT, &fmtdesc);
+		if (ret == ENOTTY)
+			return ret;
 		if (f == 0 && ret == EINVAL)
-			return -ENOSYS;
+			return ENOTTY;
 		if (ret == EINVAL)
 			break;
 		if (ret)
@@ -234,7 +239,7 @@ static int testEnumFormatsType(struct node *node, enum v4l2_buf_type type)
 		if (fmtdesc.flags & ~(V4L2_FMT_FLAG_COMPRESSED | V4L2_FMT_FLAG_EMULATED))
 			return fail("unknown flag %08x returned\n", fmtdesc.flags);
 		ret = testEnumFrameSizes(node, fmtdesc.pixelformat);
-		if (ret > 0)
+		if (ret && ret != ENOTTY)
 			return ret;
 		if (ret == 0 && !(node->caps & (V4L2_CAP_VIDEO_CAPTURE | V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE)))
 			return fail("found framesizes when no video capture is supported\n");
@@ -253,33 +258,23 @@ static int testEnumFormatsType(struct node *node, enum v4l2_buf_type type)
 
 int testEnumFormats(struct node *node)
 {
-	static const __u32 buftype2cap[] = {
-		0,
-		V4L2_CAP_VIDEO_CAPTURE,
-		V4L2_CAP_VIDEO_OUTPUT,
-		V4L2_CAP_VIDEO_OVERLAY,
-		V4L2_CAP_VBI_CAPTURE,
-		V4L2_CAP_VBI_OUTPUT,
-		V4L2_CAP_SLICED_VBI_CAPTURE,
-		V4L2_CAP_SLICED_VBI_OUTPUT,
-		V4L2_CAP_VIDEO_OUTPUT_OVERLAY,
-		V4L2_CAP_VIDEO_CAPTURE_MPLANE,
-		V4L2_CAP_VIDEO_OUTPUT_MPLANE,
-	};
-	int type;
+	bool supported = false;
+	unsigned type;
 	int ret;
 
 	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
-		ret = testEnumFormatsType(node, (enum v4l2_buf_type)type);
-		if (ret > 0)
+		ret = testEnumFormatsType(node, type);
+		if (ret && ret != ENOTTY)
 			return ret;
+		if (!ret)
+			supported = true;
 		switch (type) {
 		case V4L2_BUF_TYPE_VIDEO_CAPTURE:
 		case V4L2_BUF_TYPE_VIDEO_OUTPUT:
 		case V4L2_BUF_TYPE_VIDEO_OVERLAY:
 		case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
 		case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
-			if (ret < 0 && (node->caps & buftype2cap[type]))
+			if (ret && (node->caps & buftype2cap[type]))
 				return fail("%s cap set, but no %s formats defined\n",
 						buftype2s(type).c_str(), buftype2s(type).c_str());
 			if (!ret && !(node->caps & buftype2cap[type]))
@@ -294,18 +289,16 @@ int testEnumFormats(struct node *node)
 	}
 
 	ret = testEnumFormatsType(node, V4L2_BUF_TYPE_PRIVATE);
-	if (ret > 0)
-		return ret;
-	if (!ret)
-		warn("Buffer type PRIVATE allowed!\n");
-		
+	if (ret != ENOTTY && ret != EINVAL)
+		return fail("Buffer type PRIVATE allowed!\n");
+
 	ret = testEnumFrameSizes(node, 0x20202020);
-	if (ret >= 0)
+	if (ret != ENOTTY)
 		return fail("Accepted framesize for invalid format\n");
 	ret = testEnumFrameIntervals(node, 0x20202020, 640, 480, false);
-	if (ret >= 0)
+	if (ret != ENOTTY)
 		return fail("Accepted frameinterval for invalid format\n");
-	return 0;
+	return supported ? 0 : ENOTTY;
 }
 
 int testFBuf(struct node *node)
@@ -320,10 +313,10 @@ int testFBuf(struct node *node)
 	fbuf.fmt.priv = 0;
 	ret = doioctl(node, VIDIOC_G_FBUF, &fbuf);
 	fail_on_test(ret == 0 && !(node->caps & (V4L2_CAP_VIDEO_OVERLAY | V4L2_CAP_VIDEO_OUTPUT_OVERLAY)));
-	fail_on_test(ret == EINVAL && (node->caps & (V4L2_CAP_VIDEO_OVERLAY | V4L2_CAP_VIDEO_OUTPUT_OVERLAY)));
-	if (ret == EINVAL)
-		return -ENOSYS;
-	if (ret)
+	fail_on_test(ret == ENOTTY && (node->caps & (V4L2_CAP_VIDEO_OVERLAY | V4L2_CAP_VIDEO_OUTPUT_OVERLAY)));
+	if (ret == ENOTTY)
+		return ret;
+	if (ret && ret != EINVAL)
 		return fail("expected EINVAL, but got %d when getting framebuffer format\n", ret);
 	node->fbuf_caps = caps = fbuf.capability;
 	flags = fbuf.flags;
@@ -354,29 +347,29 @@ int testFBuf(struct node *node)
 	return 0;
 }
 
-static int testFormatsType(struct node *node, enum v4l2_buf_type type)
+static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4l2_format &fmt)
 {
 	pixfmt_set &set = node->buftype_pixfmts[type];
 	pixfmt_set *set_splane;
-	struct v4l2_format fmt;
 	struct v4l2_pix_format &pix = fmt.fmt.pix;
 	struct v4l2_pix_format_mplane &pix_mp = fmt.fmt.pix_mp;
 	struct v4l2_window &win = fmt.fmt.win;
 	struct v4l2_vbi_format &vbi = fmt.fmt.vbi;
 	struct v4l2_sliced_vbi_format &sliced = fmt.fmt.sliced;
+	unsigned min_data_samples;
+	unsigned min_sampling_rate;
+	v4l2_std_id std;
 	__u32 service_set = 0;
 	unsigned cnt = 0;
-	int ret;
-	
-	memset(&fmt, 0xff, sizeof(fmt));
-	fmt.type = type;
-	ret = doioctl(node, VIDIOC_G_FMT, &fmt);
+
+	if (ret == ENOTTY)
+		return ret;
 	if (ret == EINVAL)
-		return -ENOSYS;
+		return ENOTTY;
 	if (ret)
 		return fail("expected EINVAL, but got %d when getting format for buftype %d\n", ret, type);
 	fail_on_test(fmt.type != type);
-	
+
 	switch (type) {
 	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
@@ -387,8 +380,9 @@ static int testFormatsType(struct node *node, enum v4l2_buf_type type)
 		fail_on_test(pix.bytesperline && pix.bytesperline < pix.width);
 		fail_on_test(!pix.sizeimage);
 		fail_on_test(!pix.colorspace);
+		fail_on_test(pix.field == V4L2_FIELD_ANY);
 		if (pix.priv)
-			warn("priv is non-zero!\n");
+			return fail("priv is non-zero!\n");
 		break;
 	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
@@ -399,6 +393,7 @@ static int testFormatsType(struct node *node, enum v4l2_buf_type type)
 			return fail("unknown pixelformat %08x for buftype %d\n",
 					pix_mp.pixelformat, type);
 		fail_on_test(!pix_mp.colorspace);
+		fail_on_test(pix.field == V4L2_FIELD_ANY);
 		ret = check_0(pix_mp.reserved, sizeof(pix_mp.reserved));
 		if (ret)
 			return fail("pix_mp.reserved not zeroed\n");
@@ -415,13 +410,29 @@ static int testFormatsType(struct node *node, enum v4l2_buf_type type)
 		break;
 	case V4L2_BUF_TYPE_VBI_CAPTURE:
 	case V4L2_BUF_TYPE_VBI_OUTPUT:
-		fail_on_test(!vbi.sampling_rate);
+		// Currently VBI assumes that you have G_STD as well.
+		fail_on_test(doioctl(node, VIDIOC_G_STD, &std));
+		if (std & V4L2_STD_625_50) {
+			min_sampling_rate = 6937500;
+			// the number of databits for PAL teletext is 18 (clock run in) +
+			// 6 (framing code) + 42 * 8 (data).
+			min_data_samples = (vbi.sampling_rate * (18 + 6 + 42 * 8)) / min_sampling_rate;
+		} else {
+			min_sampling_rate = 5727272;
+			// the number of databits for NTSC teletext is 18 (clock run in) +
+			// 6 (framing code) + 34 * 8 (data).
+			min_data_samples = (vbi.sampling_rate * (18 + 6 + 34 * 8)) / min_sampling_rate;
+		}
+		fail_on_test(vbi.sampling_rate < min_sampling_rate);
 		fail_on_test(!vbi.samples_per_line);
 		fail_on_test(vbi.sample_format != V4L2_PIX_FMT_GREY);
 		fail_on_test(vbi.offset > vbi.samples_per_line);
 		ret = check_0(vbi.reserved, sizeof(vbi.reserved));
 		if (ret)
 			return fail("vbi.reserved not zeroed\n");
+		// Check that offset leaves enough room for the maximum required
+		// amount of data.
+		fail_on_test(min_data_samples > vbi.samples_per_line - vbi.offset);
 		fail_on_test(!vbi.count[0] || !vbi.count[1]);
 		fail_on_test(vbi.flags & ~(V4L2_VBI_UNSYNC | V4L2_VBI_INTERLACED));
 		if (vbi.flags & V4L2_VBI_INTERLACED)
@@ -445,10 +456,7 @@ static int testFormatsType(struct node *node, enum v4l2_buf_type type)
 		break;
 	case V4L2_BUF_TYPE_VIDEO_OVERLAY:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT_OVERLAY:
-		fail_on_test(win.field != V4L2_FIELD_ANY &&
-			     win.field != V4L2_FIELD_TOP &&
-			     win.field != V4L2_FIELD_BOTTOM &&
-			     win.field != V4L2_FIELD_INTERLACED);
+		fail_on_test(win.field == V4L2_FIELD_ANY);
 		fail_on_test(win.clipcount && !(node->fbuf_caps & V4L2_FBUF_CAP_LIST_CLIPPING));
 		for (struct v4l2_clip *clip = win.clips; clip; win.clipcount--) {
 			fail_on_test(clip == NULL);
@@ -466,33 +474,298 @@ static int testFormatsType(struct node *node, enum v4l2_buf_type type)
 	return 0;
 }
 
-int testFormats(struct node *node)
+int testGetFormats(struct node *node)
 {
+	struct v4l2_format fmt;
+	bool supported = false;
 	int type;
 	int ret;
 
 	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
-		ret = testFormatsType(node, (enum v4l2_buf_type)type);
+		memset(&fmt, 0xff, sizeof(fmt));
+		fmt.type = type;
+		ret = doioctl(node, VIDIOC_G_FMT, &fmt);
+		ret = testFormatsType(node, ret, type, fmt);
 
-		if (ret > 0)
+		if (ret && ret != ENOTTY)
 			return ret;
+		if (!ret) {
+			supported = true;
+			node->valid_buftypes |= 1 << type;
+		}
 		if (ret && (node->caps & buftype2cap[type]))
 			return fail("%s cap set, but no %s formats defined\n",
 					buftype2s(type).c_str(), buftype2s(type).c_str());
-		if (!ret && !(node->caps & buftype2cap[type]))
-			return fail("%s cap not set, but %s formats defined\n",
+		if (!ret && !(node->caps & buftype2cap[type])) {
+			switch (type) {
+			case V4L2_BUF_TYPE_VIDEO_CAPTURE:
+			case V4L2_BUF_TYPE_VIDEO_OUTPUT:
+			case V4L2_BUF_TYPE_VIDEO_OVERLAY:
+			case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+			case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+				return fail("%s cap not set, but %s formats defined\n",
 					buftype2s(type).c_str(), buftype2s(type).c_str());
+			default:
+				/* ENUMFMT doesn't support other buftypes */
+				break;
+			}
+		}
 	}
 
-	ret = testFormatsType(node, V4L2_BUF_TYPE_PRIVATE);
-	if (ret > 0)
-		return ret;
-	if (!ret)
-		warn("Buffer type PRIVATE allowed!\n");
+	memset(&fmt, 0, sizeof(fmt));
+	fmt.type = V4L2_BUF_TYPE_PRIVATE;
+	ret = doioctl(node, VIDIOC_G_FMT, &fmt);
+	if (ret != ENOTTY && ret != EINVAL)
+		return fail("Buffer type PRIVATE allowed!\n");
+	return supported ? 0 : ENOTTY;
+}
+
+int testTryFormats(struct node *node)
+{
+	struct v4l2_format fmt, fmt_try;
+	int type;
+	int ret;
+	
+	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
+		if (!(node->valid_buftypes & (1 << type)))
+			continue;
+
+		memset(&fmt, 0xff, sizeof(fmt));
+		fmt.type = type;
+		doioctl(node, VIDIOC_G_FMT, &fmt);
+		fmt_try = fmt;
+		ret = doioctl(node, VIDIOC_TRY_FMT, &fmt_try);
+		if (ret)
+			return fail("%s is valid, but no TRY_FMT was implemented\n",
+					buftype2s(type).c_str());
+		ret = testFormatsType(node, ret, type, fmt_try);
+		if (ret)
+			return ret;
+		if (memcmp(&fmt, &fmt_try, sizeof(fmt)))
+			return fail("%s: TRY_FMT(G_FMT) != G_FMT\n",
+					buftype2s(type).c_str());
+	}
+
+	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
+		if (!(node->valid_buftypes & (1 << type)))
+			continue;
+
+		memset(&fmt, 0xff, sizeof(fmt));
+		fmt.type = type;
+		fmt.fmt.pix.field = V4L2_FIELD_ANY;
+		if (type == V4L2_BUF_TYPE_VIDEO_OVERLAY ||
+		    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_OVERLAY) {
+			fmt.fmt.win.clipcount = 0;
+			fmt.fmt.win.clips = NULL;
+			fmt.fmt.win.bitmap = NULL;
+		}
+		ret = doioctl(node, VIDIOC_TRY_FMT, &fmt);
+		ret = testFormatsType(node, ret, type, fmt);
+		if (ret)
+			return fail("%s is valid, but TRY_FMT failed to return a format\n",
+					buftype2s(type).c_str());
+	}
+
+	memset(&fmt, 0, sizeof(fmt));
+	fmt.type = V4L2_BUF_TYPE_PRIVATE;
+	ret = doioctl(node, VIDIOC_TRY_FMT, &fmt);
+	if (ret != ENOTTY && ret != EINVAL)
+		return fail("Buffer type PRIVATE allowed!\n");
+	return node->valid_buftypes ? 0 : ENOTTY;
+}
+
+static int testGlobalFormat(struct node *node, int type)
+{
+	struct v4l2_fmtdesc fdesc;
+	struct v4l2_frmsizeenum fsize;
+	struct v4l2_format fmt1, fmt2;
+	struct v4l2_pix_format *p1 = &fmt1.fmt.pix;
+	struct v4l2_pix_format *p2 = &fmt2.fmt.pix;
+	struct v4l2_pix_format_mplane *mp1 = &fmt1.fmt.pix_mp;
+	struct v4l2_pix_format_mplane *mp2 = &fmt2.fmt.pix_mp;
+	__u32 pixfmt1, pixfmt2;
+	__u32 w1 = 0, w2 = 0, h1 = 0, h2 = 0;
+
+	memset(&fmt1, 0, sizeof(fmt1));
+	memset(&fmt2, 0, sizeof(fmt2));
+	fmt1.type = fmt2.type = type;
+	fdesc.index = 1;
+	fdesc.type = type;
+	memset(&fsize, 0, sizeof(fsize));
+
+	if (!doioctl(node, VIDIOC_ENUM_FMT, &fdesc)) {
+		// We found at least two different formats.
+		pixfmt2 = fdesc.pixelformat;
+		fdesc.index = 0;
+		doioctl(node, VIDIOC_ENUM_FMT, &fdesc);
+		pixfmt1 = fdesc.pixelformat;
+	} else {
+		fdesc.index = 0;
+		doioctl(node, VIDIOC_ENUM_FMT, &fdesc);
+		fsize.pixel_format = fdesc.pixelformat;
+		if (doioctl(node, VIDIOC_ENUM_FRAMESIZES, &fsize))
+			return 0;
+		pixfmt1 = pixfmt2 = fdesc.pixelformat;
+		switch (fsize.type) {
+		case V4L2_FRMSIZE_TYPE_CONTINUOUS:
+		case V4L2_FRMSIZE_TYPE_STEPWISE:
+			w1 = fsize.stepwise.min_width;
+			w2 = fsize.stepwise.max_width;
+			h1 = fsize.stepwise.min_height;
+			h2 = fsize.stepwise.max_height;
+			break;
+		case V4L2_FRMSIZE_TYPE_DISCRETE:
+			w1 = fsize.discrete.width;
+			h1 = fsize.discrete.height;
+			fsize.index = 1;
+			doioctl(node, VIDIOC_ENUM_FRAMESIZES, &fsize);
+			w2 = fsize.discrete.width;
+			h2 = fsize.discrete.height;
+			break;
+		}
+	}
+	// Check if we have found different formats, otherwise this
+	// test is pointless.
+	if (pixfmt1 == pixfmt2 && w1 == w2 && h1 == h2)
+		return 0;
+
+	if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
+	    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
+		mp1->pixelformat = pixfmt1;
+		mp1->width = w1;
+		mp1->height = h1;
+		mp2->pixelformat = pixfmt2;
+		mp2->width = w2;
+		mp2->height = h2;
+	} else {
+		p1->pixelformat = pixfmt1;
+		p1->width = w1;
+		p1->height = h1;
+		p2->pixelformat = pixfmt2;
+		p2->width = w2;
+		p2->height = h2;
+	}
+	if (doioctl(node, VIDIOC_S_FMT, &fmt1)) {
+		warn("Could not set fmt1\n");
+		return 0;
+	}
+	if (doioctl(node->node2, VIDIOC_S_FMT, &fmt2)) {
+		warn("Could not set fmt1\n");
+		return 0;
+	}
+	if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
+	    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
+		if (mp1->pixelformat == mp2->pixelformat &&
+		    mp1->width == mp2->width && mp1->height == mp2->height) {
+			// This compliance test only succeeds if the two formats
+			// are really different after S_FMT
+			warn("Could not perform global format test\n");
+			return 0;
+		}
+	} else {
+		if (p1->pixelformat == p2->pixelformat &&
+		    p1->width == p2->width && p1->height == p2->height) {
+			// This compliance test only succeeds if the two formats
+			// are really different after S_FMT
+			warn("Could not perform global format test\n");
+			return 0;
+		}
+	}
+	doioctl(node, VIDIOC_G_FMT, &fmt1);
+	if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE ||
+	    type == V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE) {
+		pixfmt1 = mp1->pixelformat;
+		w1 = mp1->width;
+		h1 = mp1->height;
+		pixfmt2 = mp2->pixelformat;
+		w2 = mp2->width;
+		h2 = mp2->height;
+	} else {
+		pixfmt1 = p1->pixelformat;
+		w1 = p1->width;
+		h1 = p1->height;
+		pixfmt2 = p2->pixelformat;
+		w2 = p2->width;
+		h2 = p2->height;
+	}
+	if (pixfmt1 != pixfmt2 || w1 != w2 || h1 != h2)
+		return fail("Global format mismatch: %08x/%dx%d vs %08x/%dx%d\n",
+				pixfmt1, w1, h1, pixfmt2, w2, h2);
+	info("Global format check succeeded for type %d\n", type);
 	return 0;
 }
 
-static int testSlicedVBICapType(struct node *node, enum v4l2_buf_type type)
+int testSetFormats(struct node *node)
+{
+	struct v4l2_format fmt, fmt_set;
+	int type;
+	int ret;
+	
+	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
+		if (!(node->valid_buftypes & (1 << type)))
+			continue;
+
+		memset(&fmt, 0xff, sizeof(fmt));
+		fmt.type = type;
+		doioctl(node, VIDIOC_G_FMT, &fmt);
+		
+		memset(&fmt_set, 0xff, sizeof(fmt_set));
+		fmt_set.type = type;
+		fmt_set.fmt.pix.field = V4L2_FIELD_ANY;
+		ret = doioctl(node, VIDIOC_S_FMT, &fmt_set);
+		ret = testFormatsType(node, ret, type, fmt_set);
+		if (ret)
+			return fail("%s is valid, but no S_FMT was implemented\n",
+					buftype2s(type).c_str());
+
+		fmt_set = fmt;
+		ret = doioctl(node, VIDIOC_S_FMT, &fmt_set);
+		ret = testFormatsType(node, ret, type, fmt_set);
+		if (ret)
+			return ret;
+		if (memcmp(&fmt, &fmt_set, sizeof(fmt)))
+			return fail("%s: S_FMT(G_FMT) != G_FMT\n",
+					buftype2s(type).c_str());
+	}
+	memset(&fmt, 0, sizeof(fmt));
+	fmt.type = V4L2_BUF_TYPE_PRIVATE;
+	ret = doioctl(node, VIDIOC_S_FMT, &fmt);
+	if (ret != ENOTTY && ret != EINVAL)
+		return fail("Buffer type PRIVATE allowed!\n");
+	if (!node->valid_buftypes)
+		return ENOTTY;
+
+	// Test if setting a format on one fh will set the format for all
+	// filehandles.
+	if (node->node2 == NULL)
+		return 0;
+	// m2m devices are unique in that the format is often per-filehandle.
+	if (node->is_m2m)
+		return 0;
+
+	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
+		switch (type) {
+		case V4L2_BUF_TYPE_VIDEO_CAPTURE:
+		case V4L2_BUF_TYPE_VIDEO_OUTPUT:
+		case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+		case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+			if (!(node->valid_buftypes & (1 << type)))
+				continue;
+
+			ret = testGlobalFormat(node, type);
+			if (ret)
+				return ret;
+			break;
+
+		default:
+			break;
+		}
+	}
+
+	return 0;
+}
+
+static int testSlicedVBICapType(struct node *node, unsigned type)
 {
 	struct v4l2_sliced_vbi_cap cap;
 	bool sliced_type = (type == V4L2_BUF_TYPE_SLICED_VBI_CAPTURE ||
@@ -504,18 +777,14 @@ static int testSlicedVBICapType(struct node *node, enum v4l2_buf_type type)
 	memset(&cap.reserved, 0, sizeof(cap.reserved));
 	cap.type = type;
 	ret = doioctl(node, VIDIOC_G_SLICED_VBI_CAP, &cap);
+	if (ret == ENOTTY || ret == EINVAL) {
+		fail_on_test(sliced_type && (node->caps & buftype2cap[type]));
+		return ret == ENOTTY ? ret : 0;
+	}
+	fail_on_test(ret);
 	fail_on_test(check_0(cap.reserved, sizeof(cap.reserved)));
 	fail_on_test(cap.type != type);
-	fail_on_test(ret && ret != EINVAL && sliced_type);
-	if (ret == EINVAL) {
-		fail_on_test(sliced_type && (node->caps & buftype2cap[type]));
-		if (node->caps & (V4L2_CAP_SLICED_VBI_CAPTURE | V4L2_CAP_SLICED_VBI_OUTPUT))
-			return 0;
-		return -ENOSYS;
-	}
-	if (ret)
-		return fail("expected EINVAL, but got %d when getting sliced VBI caps buftype %d\n", ret, type);
-	fail_on_test(!(node->caps & buftype2cap[type]));
+	fail_on_test(!sliced_type || !(node->caps & buftype2cap[type]));
 
 	for (int f = 0; f < 2; f++)
 		for (int i = 0; i < 24; i++)
@@ -537,3 +806,110 @@ int testSlicedVBICap(struct node *node)
 		return ret;
 	return testSlicedVBICapType(node, V4L2_BUF_TYPE_VIDEO_CAPTURE);
 }
+
+static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
+{
+	struct v4l2_captureparm *cap = &parm.parm.capture;
+	struct v4l2_outputparm *out = &parm.parm.output;
+	int ret;
+
+	switch (parm.type) {
+	case V4L2_BUF_TYPE_VIDEO_CAPTURE:
+	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
+		ret = check_0(cap->reserved, sizeof(cap->reserved));
+		if (ret)
+			return fail("reserved not zeroed\n");
+		fail_on_test(cap->readbuffers > VIDEO_MAX_FRAME);
+		if (!(node->caps & V4L2_CAP_READWRITE))
+			fail_on_test(cap->readbuffers);
+		else if (node->caps & V4L2_CAP_STREAMING)
+			fail_on_test(!cap->readbuffers);
+		fail_on_test(cap->capability & ~V4L2_CAP_TIMEPERFRAME);
+		fail_on_test(cap->capturemode & ~V4L2_MODE_HIGHQUALITY);
+		if (cap->capturemode & V4L2_MODE_HIGHQUALITY)
+			warn("V4L2_MODE_HIGHQUALITY is poorly defined\n");
+		fail_on_test(cap->extendedmode);
+		if (cap->capability & V4L2_CAP_TIMEPERFRAME)
+			fail_on_test(cap->timeperframe.numerator == 0 || cap->timeperframe.denominator == 0);
+		break;
+	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
+	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
+		ret = check_0(out->reserved, sizeof(out->reserved));
+		if (ret)
+			return fail("reserved not zeroed\n");
+		fail_on_test(out->writebuffers > VIDEO_MAX_FRAME);
+		if (!(node->caps & V4L2_CAP_READWRITE))
+			fail_on_test(out->writebuffers);
+		else if (node->caps & V4L2_CAP_STREAMING)
+			fail_on_test(!out->writebuffers);
+		fail_on_test(out->capability & ~V4L2_CAP_TIMEPERFRAME);
+		fail_on_test(out->outputmode);
+		fail_on_test(out->extendedmode);
+		if (out->capability & V4L2_CAP_TIMEPERFRAME)
+			fail_on_test(out->timeperframe.numerator == 0 || out->timeperframe.denominator == 0);
+		break;
+	default:
+		break;
+	}
+	return 0;
+}
+
+static int testParmType(struct node *node, unsigned type)
+{
+	struct v4l2_streamparm parm;
+	int ret;
+
+	memset(&parm, 0, sizeof(parm));
+	parm.type = type;
+	ret = doioctl(node, VIDIOC_G_PARM, &parm);
+	if (ret == ENOTTY)
+		return ret;
+	if (ret == EINVAL)
+		return ENOTTY;
+	if (ret)
+		return fail("expected EINVAL, but got %d when getting parms for buftype %d\n", ret, type);
+	fail_on_test(parm.type != type);
+	ret = testParmStruct(node, parm);
+	if (ret)
+		return ret;
+
+	memset(&parm, 0, sizeof(parm));
+	parm.type = type;
+	ret = doioctl(node, VIDIOC_S_PARM, &parm);
+	if (ret == ENOTTY)
+		return 0;
+	if (ret)
+		return fail("got error %d when setting parms for buftype %d\n", ret, type);
+	fail_on_test(parm.type != type);
+	return testParmStruct(node, parm);
+}
+
+int testParm(struct node *node)
+{
+	bool supported = false;
+	int type;
+	int ret;
+
+	for (type = 0; type <= V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE; type++) {
+		ret = testParmType(node, type);
+
+		if (ret && ret != ENOTTY)
+			return ret;
+		if (!ret) {
+			supported = true;
+			if (type != V4L2_BUF_TYPE_VIDEO_CAPTURE &&
+			    type != V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE &&
+			    type != V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE &&
+			    type != V4L2_BUF_TYPE_VIDEO_OUTPUT)
+				return fail("G/S_PARM is only allowed for video capture/output\n");
+			if (!(node->caps & buftype2cap[type]))
+				return fail("%s cap not set, but G/S_PARM worked\n",
+						buftype2s(type).c_str());
+		}
+	}
+
+	ret = testParmType(node, V4L2_BUF_TYPE_PRIVATE);
+	if (ret != ENOTTY && ret != EINVAL)
+		return fail("Buffer type PRIVATE allowed!\n");
+	return supported ? 0 : ENOTTY;
+}
