@@ -13,11 +13,10 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
 #include <unistd.h>
-#include <features.h>		/* Uses _GNU_SOURCE to define getsubopt in stdlib.h */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -34,7 +33,6 @@
 #include <sys/klog.h>
 
 #include <linux/videodev2.h>
-#include <linux/i2c-id.h>
 #include <media/v4l2-chip-ident.h>
 
 #include <list>
@@ -540,7 +538,10 @@ int main(int argc, char **argv)
 		printf("\tDriver name   : %s\n", vcap.driver);
 		printf("\tCard type     : %s\n", vcap.card);
 		printf("\tBus info      : %s\n", vcap.bus_info);
-		printf("\tDriver version: %d\n", vcap.version);
+		printf("\tDriver version: %d.%d.%d\n",
+				vcap.version >> 16,
+				(vcap.version >> 8) & 0xff,
+				vcap.version & 0xff);
 		printf("\tCapabilities  : 0x%08X\n", vcap.capabilities);
 		printf("%s", cap2s(vcap.capabilities).c_str());
 	}
@@ -621,8 +622,6 @@ int main(int argc, char **argv)
 	}
 
 	if (options[OptGetRegister]) {
-		int stride = 1;
-
 		get_reg.match = match;
 		printf("ioctl: VIDIOC_DBG_G_REGISTER\n");
 
